"use strict";
/* global browser, checkContentFilters, checkSplitHorizon */


const baseStudySetup = {
  activeExperimentName: browser.runtime.id,
  studyType: "shield",
  // telemetry
  telemetry: {
    // default false. Actually send pings.
    send: true,
    // Marks pings with testing=true.  Set flag to `true` before final release
    removeTestingFlag: true,
  },
  endings: {
    // standard endings
    "user-disable": {
      baseUrls: [],
      category: "ended-negative",
    },
    ineligible: {
      baseUrls: [],
      category: "ended-neutral",
    },
    expired: {
      baseUrls: [],
      category: "ended-positive",
    },
  },
  weightedVariations: [
    {
      name: "study-active",
      weight: 1.5
    }
  ],
  // maximum time that the study should run, from the first run
  expire: {
    days: 7,
  },
  allowEnroll: true,
};


const stateManager = {
  _captiveState: "unknown",

  set captiveState(captiveState) {
    this._captiveState = captiveState;
  },

  get captiveState() {
    return this._captiveState;
  }
};


// Entry point for sending Telemetry data from local storage
async function telemetry() {
  let {contentFilterData} = await browser.storage.local.get("contentFilterData");
  let {splitHorizonData} = await browser.storage.local.get("splitHorizonData");
  await browser.storage.local.remove(["contentFilterData", "splitHorizonData"]);

  // Send Telemetry data
  if (contentFilterData && splitHorizonData) {
    // Read data from local storage
    let usesComcastMalwareFilter = String(contentFilterData.usesComcastMalwareFilter);
    let usesComcastParentalFilter = String(contentFilterData.usesComcastParentalFilter);
    let blocksExampleAdultSite = String(contentFilterData.blocksExampleAdultSite);
    let usesGoogleSafeSearch = String(contentFilterData.usesGoogleSafeSearch);
    let usesYouTubeSafeSearch = String(contentFilterData.usesYouTubeSafeSearch);
    let notInPSL = String(splitHorizonData.notInPSL.size);
    let pslResolvesLocal = String(splitHorizonData.pslResolvesLocal.size);
    let uniqueDomains = String(splitHorizonData.uniqueDomains.size);
    let telemetryData = {"usesComcastMalwareFilter": usesComcastMalwareFilter,
                         "usesComcastParentalFilter": usesComcastParentalFilter,
                         "blocksExampleAdultSite": blocksExampleAdultSite,
                         "usesGoogleSafeSearch": usesGoogleSafeSearch,
                         "usesYouTubeSafeSearch": usesYouTubeSafeSearch,
                         "notInPSL": notInPSL,
                         "pslResolvesLocal": pslResolvesLocal,
                         "uniqueDomains": uniqueDomains};

    // Send Telemety data, and check copies of pings on disk
    await browser.study.sendTelemetry(telemetryData);
  }
}


// Entry point for running measurements
async function measure() {
  // Check for content filters by: 
  //   a) resolving canary domains 
  //   b) resolving "safe search" domains for YouTube, Google, and Bing
  //   c) resolving domain(s) that might appear on a blocklist, 
  //   i.e. exampleadultsite.com
  await checkContentFilters();
  
  // Check for split horizon by listening for each 
  // request and seeing if either: 
  //   a) the URL doesn't have a public suffix
  //   b) the URL has a public suffix but resolves to an RFC 1918 address
  let filter = {urls: ["<all_urls>"], types: ["main_frame"]};
  browser.webRequest.onBeforeRedirect.addListener(checkSplitHorizon, filter);
  browser.webRequest.onCompleted.addListener(checkSplitHorizon, filter);
}


async function telemetry_and_measure(details) {
  let currentlyOffline = (stateManager.captiveState !== "unlocked_portal" &&
                          stateManager.captiveState !== "not_captive");
  let goingOnline = (details.state === "unlocked_portal" || 
                     details.state === "not_captive");

  if (currentlyOffline && goingOnline) {
    browser.study.sendTelemetry({stateKey: "loaded"});
    await telemetry();
    await measure();
  }
  stateManager.captiveState = details.state;
}


const rollout = {
  async init() {
    browser.study.onEndStudy.addListener((ending) => {
      console.log("Study ending", ending);
      browser.management.uninstallSelf();
    });
    browser.study.onReady.addListener(() => {
      this.onReady();
    });
    await browser.study.setup(baseStudySetup);
  },


  async onReady() {
    // Return early as we don't have a control.json file
    const studyInfo = await browser.study.getStudyInfo();
    const variation = studyInfo.variation.name;
    if (variation === "control") {
      return;
    }
    if (studyInfo.isFirstRun) {
      browser.study.sendTelemetry({stateKey: "installed"});
    }

    // If the captive portal is already unlocked or doesn't exist,
    // run the measurement
    let captiveState = await browser.captivePortal.getState();
    if (captiveState === "unlocked_portal" || captiveState === "not_captive") {
      await telemetry_and_measure({state: captiveState});
    }

    // Listen to the captive portal when it unlocks
    browser.captivePortal.onStateChanged.addListener(telemetry_and_measure);
  }
};


rollout.init();
